using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using MacGruber;
using MeshVR;
using SimpleJSON;
using UnityEngine;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;

namespace CheesyFX
{
    public class EmoteManager
    {
        public static JSONStorableBool enabled = new JSONStorableBool("Emotes Enabled", true);
        private static List<object> UIElements = new List<object>();
        private static UIDynamicTabBar tabbar;
        private static int lastTabId;
        
        private static Request particleBundle;
        public static StimSprayer stimulationEmotes;
        public static OrgasmSprayer orgasmEmotes;
        public static OrgasmFadeSprayer orgasmFadeEmotes;

        private static bool ready;

        public static void Init()
        {
            Request request = new AssetLoader.AssetBundleFromFileRequest {path = ReadMyLips.packageUid+"Custom/Scripts/CheesyFX/BodyLanguage/assets/particle.assetbundle", callback = OnBundleLoaded};
            AssetLoader.QueueLoadAssetBundleFromFile(request);
            stimulationEmotes = new StimSprayer();
            orgasmEmotes = new OrgasmSprayer();
            orgasmFadeEmotes = new OrgasmFadeSprayer();
        }
        
        private static void OnBundleLoaded(Request request)
        {
            try
            {
                particleBundle = request;
                var head = ReadMyLips.atom.forceReceivers.First(x => x.name == "head").transform;
                var go = request.assetBundle.LoadAsset<GameObject>("assets/particle/gameobject.prefab");
                stimulationEmotes.Init(go);
                orgasmEmotes.Init(go);
                orgasmFadeEmotes.Init(go);
                enabled.setCallbackFunction += val => stimulationEmotes.emission.enabled = val;
                ready = true;
            }
            catch (Exception e)
            {
                SuperController.LogError(e.ToString());
            }
        }

        public static void SetActive(bool val)
        {
            stimulationEmotes.enabled.val = val;
            orgasmEmotes.enabled.val = val;
            orgasmFadeEmotes.enabled.val = val;
        }

        public static void CreateUI()
        {
            ReadMyLips.singleton.presetSystem.CreateUI();
            UIDynamicButton button;
            button = ReadMyLips.singleton.CreateButton("Return");
            button.buttonColor = new Color(0.55f, 0.90f, 1f);
            button.button.onClick.AddListener(
                () =>
                {
                    // Utils.RemoveUIElements(ReadMyLips.singleton, UIElements);
                    ReadMyLips.singleton.ClearUI();
                    // panelOpen = false;
                    ReadMyLips.singleton.CreateUI();
                });
            // UIElements.Add(button);
            var spacer = ReadMyLips.singleton.CreateSpacer(true);
            spacer.height = 50f;
            // UIElements.Add(spacer);
            
            tabbar = UIManager.CreateTabBar(new [] {"Stimulation", "Orgasm", "OrgasmFade"}, SelectTab, script:ReadMyLips.singleton);
            tabbar.SelectTab(lastTabId);
            
        }

        private static void SelectTab(int id)
        {
            Utils.RemoveUIElements(ReadMyLips.singleton, UIElements);
            lastTabId = id;
            switch (id)
            {
                case 0:
                {
                    stimulationEmotes.CreateUI(UIElements);
                    break;
                }
                case 1: 
                {
                    orgasmEmotes.CreateUI(UIElements);
                    break;
                }
                case 2: 
                {
                    orgasmFadeEmotes.CreateUI(UIElements);
                    break;
                }
            }
        }

        public static void Destroy()
        {
            AssetLoader.DoneWithAssetBundleFromFile(particleBundle.path);
            stimulationEmotes.Destroy();
            orgasmEmotes.Destroy();
            orgasmFadeEmotes.Destroy();
        }

        public static JSONClass Store()
        {
            var jc = new JSONClass();
            enabled.Store(jc);
            jc["StimulationEmotes"] = stimulationEmotes.Store();
            jc["OrgasmEmotes"] = orgasmEmotes.Store();
            jc["OrgasmFadeEmotes"] = orgasmFadeEmotes.Store();
            return jc;
        }
        
        public static void Load(JSONClass jc)
        {
            if (ready)
            {
                enabled.Load(jc);
                stimulationEmotes.Load(jc["StimulationEmotes"].AsObject);
                orgasmEmotes.Load(jc["OrgasmEmotes"].AsObject);
                orgasmFadeEmotes.Load(jc["OrgasmFadeEmotes"].AsObject);
            }
            else DeferredLoad(jc).Start();
        }

        private static IEnumerator DeferredLoad(JSONClass jc)
        {
            while (!ready) yield return new WaitForSeconds(.1f);
            Load(jc);
        }
    }
    
    internal static class ColorExt
    {
        public static HSVColor ToHSV(this Color color)
        {
            float h, s, v;
            Color.RGBToHSV(color, out h, out s, out v);
            return new HSVColor
            {
                H = h,
                S = s,
                V = v
            };
        }
        
        public static Color ToRGB(this HSVColor hsv)
        {
            float h, s, v;
            return Color.HSVToRGB(hsv.H, hsv.S, hsv.V);
        }
    }
}